@@ -35,4 +35,5 @@ sshuttle.helpers.verbose = verbosity
 
 import sshuttle.cmdline_options as options
 from sshuttle.server import main
-main(options.latency_control, options.auto_hosts, options.to_nameserver)
+main(options.latency_control, options.auto_hosts, options.to_nameserver,
+     options.auto_nets)
